import { Observable } from 'rxjs/Observable';
import { map } from 'rxjs/operator/map';
import { mergeMap } from 'rxjs/operator/mergeMap';
import { toArray } from 'rxjs/operator/toArray';
import { Injectable, Injector, NgModule, ReflectiveInjector } from '@angular/core';
import { auth, database, initializeApp } from 'firebase';
import * as firebase from 'firebase';
import { ReplaySubject } from 'rxjs/ReplaySubject';
import { fromPromise } from 'rxjs/observable/fromPromise';
import { of } from 'rxjs/observable/of';
import { mapTo } from 'rxjs/operator/mapTo';

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var DataSnapshotObservable = (function (_super) {
    __extends(DataSnapshotObservable, _super);
    function DataSnapshotObservable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DataSnapshotObservable.prototype.exists = function () {
        return map.call(this, function (snapshot) { return snapshot.exists(); });
    };
    DataSnapshotObservable.prototype.children = function () {
        return map.call(this, function (snapshot) { return new DataSnapshotObservable(function (sub) {
            snapshot.forEach(function (childSnapshot) {
                sub.next(childSnapshot);
                return false;
            });
            sub.complete();
        }); });
    };
    /**
     * This operator takes the result of .val() for all children of the snapshot and emits
     * them as an array.
     * Contents of source snapshot:
     * ```
     * {
     *  childA: { prop: 'Hello' },
     *  childB: { prop: 'World!' },
     * }
     * ```
     * Result of operator:
     * ```
     * [
     *  { prop: 'Hello' },
     *  { prop: 'World!' },
     * ]
     * ```
     * @returns {Observable<C[]>}
     */
    DataSnapshotObservable.prototype.toValArray = function () {
        return mergeMap.call(this.children(), function (children) { return toArray.call(children.val()); });
    };
    DataSnapshotObservable.prototype.values = function () {
        return this.toValArray();
    };
    DataSnapshotObservable.prototype.keys = function () {
        return mergeMap.call(this.children(), function (children) { return toArray.call(children.key()); });
    };
    DataSnapshotObservable.prototype.list = function () {
        return mergeMap.call(this.children(), function (children) { return toArray.call(children.entry()); });
    };
    DataSnapshotObservable.prototype.entry = function () {
        return map.call(this, function (snapshot) { return ({ val: snapshot.val(), key: snapshot.key }); });
    };
    DataSnapshotObservable.prototype.key = function () {
        return map.call(this, function (snapshot) { return snapshot.key; });
    };
    /**
     * When listening to events such as {@link Event.ChildMoved} the snapshot includes
     * the key of the child before this snapshots one. This operator maps to this key.
     * @returns {Observable<string>}
     */
    DataSnapshotObservable.prototype.prevKey = function () {
        return map.call(this, function (snapshot) { return snapshot.prevKey; });
    };
    DataSnapshotObservable.prototype.val = function () {
        return map.call(this, function (snapshot) { return snapshot.val(); });
    };
    DataSnapshotObservable.prototype.getPriority = function () {
        return map.call(this, function (snapshot) { return snapshot.getPriority(); });
    };
    DataSnapshotObservable.prototype.exportVal = function () {
        return map.call(this, function (snapshot) { return snapshot.exportVal(); });
    };
    DataSnapshotObservable.prototype.hasChild = function (path) {
        return map.call(this, function (snapshot) { return snapshot.hasChild(path); });
    };
    DataSnapshotObservable.prototype.hasChildren = function () {
        return map.call(this, function (snapshot) { return snapshot.hasChildren(); });
    };
    DataSnapshotObservable.prototype.numChildren = function () {
        return map.call(this, function (snapshot) { return snapshot.numChildren(); });
    };
    DataSnapshotObservable.prototype.child = function (path) {
        var _this = this;
        return new DataSnapshotObservable(function (sub) {
            var subscription = map.call(_this, function (snapshot) { return snapshot.child(path); })
                .subscribe(sub);
            return function () { return subscription.unsubscribe(); };
        });
    };
    return DataSnapshotObservable;
}(Observable));

function wrapExternalPromise(promise) {
    return new Promise(function (resolve, reject) {
        promise
            .then(function (res) { return resolve(res); })
            .catch(function (err) { return reject(err); });
    });
}
function wrapPromise(promiseFact) {
    if (invokeLazy()) {
        return new Observable(function (sub) {
            fromPromise(wrapExternalPromise(promiseFact()))
                .subscribe(sub);
        });
    }
    return fromPromise(wrapExternalPromise(promiseFact()));
}

var FirebaseUserCredential = (function () {
    function FirebaseUserCredential(cred) {
        this.credential = cred.credential || undefined;
        this.user = cred.user ? new FirebaseUser(cred.user) : undefined;
    }
    return FirebaseUserCredential;
}());
var FirebaseUser = (function () {
    function FirebaseUser(user) {
        this.user = user;
    }
    Object.defineProperty(FirebaseUser.prototype, "displayName", {
        get: function () {
            return this.user.displayName;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "email", {
        get: function () {
            return this.user.email;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "emailVerified", {
        get: function () {
            return this.user.emailVerified;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "isAnonymous", {
        get: function () {
            return this.user.isAnonymous;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "photoURL", {
        get: function () {
            return this.user.photoURL;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "providerData", {
        get: function () {
            return this.user.providerData;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "providerId", {
        get: function () {
            return this.user.providerId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "refreshToken", {
        get: function () {
            return this.user.refreshToken;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirebaseUser.prototype, "uid", {
        get: function () {
            return this.user.uid;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @returns {Observable<void>} - Returns {@link DeleteUserError} if operation fails.
     */
    FirebaseUser.prototype.delete = function () {
        var _this = this;
        return wrapPromise(function () { return _this.user.delete(); });
    };
    FirebaseUser.prototype.getToken = function (forceRefresh) {
        var _this = this;
        return wrapPromise(function () { return _this.user.getToken(forceRefresh); });
    };
    /**
     * @returns {Observable<FirebaseUser>} - Returns {@link LinkUserError} if operation fails.
     */
    FirebaseUser.prototype.link = function (credential) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.user.link(credential); }), function (user) { return new FirebaseUser(user); });
    };
    /**
     * @returns {Observable<FirebaseUserCredential>} - Returns {@link LinkUserWithPopupError} if
     *     operation fails.
     */
    FirebaseUser.prototype.linkWithPopup = function (provider) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.user.linkWithPopup(provider); }), function (cred) { return new FirebaseUserCredential(cred); });
    };
    /**
     * @returns {Observable<FirebaseUserCredential>} - Returns {@link LinkUserWithRedirectError} if
     *     operation fails.
     */
    FirebaseUser.prototype.linkWithRedirect = function (provider) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.user.linkWithRedirect(provider); }), function (cred) { return new FirebaseUserCredential(cred); });
    };
    /**
     * @returns {Observable<void>} - Returns {@link ReauthenticateError} if operation
     * fails.
     */
    FirebaseUser.prototype.reauthenticate = function (credential) {
        var _this = this;
        return wrapPromise(function () { return _this.user.reauthenticate(credential); });
    };
    FirebaseUser.prototype.reload = function () {
        var _this = this;
        return wrapPromise(function () { return _this.user.reload(); });
    };
    FirebaseUser.prototype.sendEmailVerification = function () {
        var _this = this;
        return wrapPromise(function () { return _this.user.sendEmailVerification(); });
    };
    FirebaseUser.prototype.unlink = function (providerId) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.user.unlink(providerId); }), function (user) { return new FirebaseUser(user); });
    };
    /**
     * @returns {Observable<void>} - Returns {@link UpdateEmailError} if operation
     * fails.
     */
    FirebaseUser.prototype.updateEmail = function (newEmail) {
        var _this = this;
        return wrapPromise(function () { return _this.user.updateEmail(newEmail); });
    };
    /**
     * @returns {Observable<void>} - Returns {@link UpdatePasswordError} if operation
     * fails.
     */
    FirebaseUser.prototype.updatePassword = function (newPassword) {
        var _this = this;
        return wrapPromise(function () { return _this.user.updatePassword(newPassword); });
    };
    FirebaseUser.prototype.updateProfile = function (profile) {
        var _this = this;
        return wrapPromise(function () { return _this.user.updateProfile(profile); });
    };
    return FirebaseUser;
}());

var NativeFirebaseAuth = (function () {
    function NativeFirebaseAuth() {
    }
    return NativeFirebaseAuth;
}());
var NativeFirebaseDatabase = (function () {
    function NativeFirebaseDatabase() {
    }
    return NativeFirebaseDatabase;
}());

function runInZone() {
    var _this = this;
    var z = Zone.current;
    return new this.constructor(function (sub) {
        var subscription = _this.subscribe(z.wrap(sub.next, 'Observable.next').bind(sub), z.wrap(sub.error, 'Observable.error').bind(sub), z.wrap(sub.complete, 'Observable.complete').bind(sub));
        return function () { return subscription.unsubscribe(); };
    });
}

var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var FirebaseAuth = (function () {
    function FirebaseAuth(fbAuth) {
        this.fbAuth = fbAuth;
        this.$user = new ReplaySubject(1);
        var onAuthStateChangedObs = new Observable(function (sub) { return fbAuth.onAuthStateChanged(function (user) { return sub.next(user ? new FirebaseUser(user) : null); }, function (err) { return sub.error(err); }, function () { return sub.complete(); }); });
        runInZone.call(onAuthStateChangedObs)
            .subscribe(this.$user);
    }
    /**
     * @param code
     * @returns {Observable<void>} - Returns {@link ActionCodeError} if operation fails.
     */
    FirebaseAuth.prototype.applyActionCode = function (code) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.applyActionCode(code); });
    };
    /**
     * @param code
     * @returns {Observable<ActionCodeInfo>} - Returns {@link ActionCodeError} if operation fails.
     */
    FirebaseAuth.prototype.checkActionCode = function (code) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.checkActionCode(code); });
    };
    /**
     * @param code
     * @param newPassword
     * @returns {Observable<void>} - Returns {@link ConfirmPasswordResetError} if operation fails.
     */
    FirebaseAuth.prototype.confirmPasswordReset = function (code, newPassword) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.confirmPasswordReset(code, newPassword); });
    };
    /**
     *
     * @param email
     * @param password
     * @returns {Observable<FirebaseUser>} - Returns {@link CreateUserWithEmailAndPasswordError} if
     *     operation fails.
     */
    FirebaseAuth.prototype.createUserWithEmailAndPassword = function (email, password) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.createUserWithEmailAndPassword(email, password); }), function (user) { return new FirebaseUser(user); });
    };
    /**
     *
     * @param email
     * @returns {Observable<string[]>} - Returns {@link FetchProvidersForEmailError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.fetchProvidersForEmail = function (email) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.fetchProvidersForEmail(email); });
    };
    /**
     * @returns {Observable<FirebaseUserCredential>} - Returns {@link GetRedirectResultError} if
     *     operation fails.
     */
    FirebaseAuth.prototype.getRedirectResult = function () {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.getRedirectResult(); }), function (cred) { return new FirebaseUserCredential(cred); });
    };
    /**
     * @param email
     * @returns {Observable<void>} - Returns {@link SendPasswordResetEmailError} if operation fails.
     */
    FirebaseAuth.prototype.sendPasswordResetEmail = function (email) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.sendPasswordResetEmail(email); });
    };
    /**
     *
     * @returns {Observable<FirebaseUser>} - Returns {@link SignInAnonymouslyError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.signInAnonymously = function () {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.signInAnonymously(); }), function (user) { return new FirebaseUser(user); });
    };
    /**
     * @param credential
     * @returns {Observable<FirebaseUser>} - Returns {@link SignInWithCredentialError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.signInWithCredential = function (credential) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.signInWithCredential(credential); }), function (user) { return new FirebaseUser(user); });
    };
    /**
     * @param token
     * @returns {Observable<FirebaseUser>} - Returns {@link SignInWithCustomTokenError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.signInWithCustomToken = function (token) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.signInWithCustomToken(token); }), function (user) { return new FirebaseUser(user); });
    };
    /**
     * @param email
     * @param password
     * @returns {Observable<FirebaseUser>} - Returns {@link SignInWithEmailAndPasswordError} if
     *     operation fails.
     */
    FirebaseAuth.prototype.signInWithEmailAndPassword = function (email, password) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.signInWithEmailAndPassword(email, password); }), function (user) { return new FirebaseUser(user); });
    };
    /**
     * @param provider
     * @returns {Observable<FirebaseUser>} - Returns {@link SignInWithPopupError} if operation fails.
     */
    FirebaseAuth.prototype.signInWithPopup = function (provider) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.signInWithPopup(provider); }), function (cred) { return new FirebaseUserCredential(cred); });
    };
    /**
     * @param provider
     * @returns {Observable<FirebaseUser>} - Returns {@link SignInWithRedirectError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.signInWithRedirect = function (provider) {
        var _this = this;
        return map.call(wrapPromise(function () { return _this.fbAuth.signInWithRedirect(provider); }), function (cred) { return new FirebaseUserCredential(cred); });
    };
    FirebaseAuth.prototype.signOut = function () {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signOut(); });
    };
    /**
     *
     * @param code
     * @returns {Observable<string>} - Returns {@link VerifyPasswordResetCodeError} if operation
     * fails.
     */
    FirebaseAuth.prototype.verifyPasswordResetCode = function (code) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.verifyPasswordResetCode(code); });
    };
    return FirebaseAuth;
}());
FirebaseAuth = __decorate$1([
    Injectable(),
    __metadata("design:paramtypes", [NativeFirebaseAuth])
], FirebaseAuth);

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate$2 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$1 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * Enum of event types.
 */
var Event = (function () {
    function Event() {
    }
    return Event;
}());
Event.Value = 'value';
Event.ChildAdded = 'child_added';
Event.ChildChanged = 'child_changed';
Event.ChildRemoved = 'child_removed';
Event.ChildMoved = 'child_moved';
var FirebaseQuery = (function () {
    function FirebaseQuery(_ref) {
        this._ref = _ref;
    }
    Object.defineProperty(FirebaseQuery.prototype, "ref", {
        get: function () {
            return this.wrappedRef;
        },
        enumerable: true,
        configurable: true
    });
    FirebaseQuery.prototype.orderByChild = function (child) {
        this._call('orderByChild', child);
        return this;
    };
    FirebaseQuery.prototype.orderByKey = function () {
        this._call('orderByKey');
        return this;
    };
    FirebaseQuery.prototype.orderByPriority = function () {
        this._call('orderByPriority');
        return this;
    };
    FirebaseQuery.prototype.orderByValue = function () {
        this._call('orderByValue');
        return this;
    };
    FirebaseQuery.prototype.startAt = function (value, key) {
        this._call('startAt', value, key);
        return this;
    };
    FirebaseQuery.prototype.endAt = function (value, key) {
        this._call('endAt', value, key);
        return this;
    };
    FirebaseQuery.prototype.equalTo = function (value, key) {
        this._call('equalTo', value, key);
        return this;
    };
    FirebaseQuery.prototype.limitToFirst = function (limit) {
        this._call('limitToFirst', limit);
        return this;
    };
    FirebaseQuery.prototype.limitToLast = function (limit) {
        this._call('limitToLast', limit);
        return this;
    };
    FirebaseQuery.prototype.once = function (event) {
        var _this = this;
        return runInZone.call(new DataSnapshotObservable(function (sub) {
            _this.getQueryOrRef().once(event, _this.getEventHandler(sub, true), function (err) { return function () { return sub.error(err); }; });
        }));
    };
    FirebaseQuery.prototype.onceValue = function () {
        return this.once('value');
    };
    FirebaseQuery.prototype.onceChildAdded = function () {
        return this.once('child_added');
    };
    FirebaseQuery.prototype.onceChildChanged = function () {
        return this.once('child_changed');
    };
    FirebaseQuery.prototype.onceChildMoved = function () {
        return this.once('child_moved');
    };
    FirebaseQuery.prototype.onceChildRemoved = function () {
        return this.once('child_removed');
    };
    FirebaseQuery.prototype.on = function (event) {
        var _this = this;
        return runInZone.call(new DataSnapshotObservable(function (sub) {
            var cb = _this.getQueryOrRef().on(event, _this.getEventHandler(sub), function (err) { return sub.error(err); });
            return function () { return _this.getQueryOrRef().off(event, cb); };
        }));
    };
    FirebaseQuery.prototype.onValue = function () {
        return this.on('value');
    };
    FirebaseQuery.prototype.onChildAdded = function () {
        return this.on('child_added');
    };
    FirebaseQuery.prototype.onChildChanged = function () {
        return this.on('child_changed');
    };
    FirebaseQuery.prototype.onChildMoved = function () {
        return this.on('child_moved');
    };
    FirebaseQuery.prototype.onChildRemoved = function () {
        return this.on('child_removed');
    };
    FirebaseQuery.prototype.isEqual = function (query) {
        return this.getQueryOrRef().isEqual(query.getQueryOrRef());
    };
    FirebaseQuery.prototype.getEventHandler = function (sub, complete) {
        return function (snapshot, prevKey) {
            snapshot.prevKey = prevKey;
            sub.next(snapshot);
            if (complete) {
                sub.complete();
            }
        };
    };
    FirebaseQuery.prototype.getQueryOrRef = function () {
        if (this.query) {
            return this.query;
        }
        return this._ref;
    };
    FirebaseQuery.prototype._call = function (fnName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (this.query) {
            this.query = (_a = this.query)[fnName].apply(_a, args);
        }
        else {
            this.query = (_b = this._ref)[fnName].apply(_b, args);
        }
        var _a, _b;
    };
    return FirebaseQuery;
}());
var FirebaseDatabaseRefConfig = (function () {
    function FirebaseDatabaseRefConfig(parent, ref) {
        this.parent = parent;
        this.ref = ref;
    }
    return FirebaseDatabaseRefConfig;
}());
var FirebaseDatabaseRef = (function (_super) {
    __extends$1(FirebaseDatabaseRef, _super);
    function FirebaseDatabaseRef(config) {
        var _this = _super.call(this, config.ref) || this;
        _this.config = config;
        _this.wrappedRef = _this;
        return _this;
    }
    FirebaseDatabaseRef.create = function (parent, ref) {
        return new FirebaseDatabaseRef(new FirebaseDatabaseRefConfig(parent, ref));
    };
    Object.defineProperty(FirebaseDatabaseRef.prototype, "key", {
        get: function () {
            return this._ref.key;
        },
        enumerable: true,
        configurable: true
    });
    FirebaseDatabaseRef.prototype.child = function (path) {
        return FirebaseDatabaseRef.create(this, this._ref.child(path));
    };
    FirebaseDatabaseRef.prototype.set = function (value) {
        var _this = this;
        return wrapPromise(function () { return _this._ref.set(value); });
    };
    FirebaseDatabaseRef.prototype.setPriority = function (priority) {
        var _this = this;
        // There seems to be a bug with the typing for #setPriority(priority, onComplete): Promise
        // The firebase library, in every other case, declares the onComplete function optional since a
        // Promise is returned as well.
        return wrapPromise(function () { return _this._ref.setPriority(priority); });
    };
    FirebaseDatabaseRef.prototype.setWithPriority = function (newVal, priority) {
        var _this = this;
        return wrapPromise(function () { return _this._ref.setWithPriority(newVal, priority); });
    };
    FirebaseDatabaseRef.prototype.push = function (value) {
        var pushRef = this._ref.push(value);
        var ref = FirebaseDatabaseRef.create(this, pushRef);
        // Only if a value to push was given, use ref as promise, since otherwise
        // pushRef.then will be undefined
        if (value) {
            return mapTo.call(wrapPromise(function () { return pushRef; }), ref);
        }
        return of(ref);
    };
    FirebaseDatabaseRef.prototype.update = function (value) {
        var _this = this;
        return wrapPromise(function () { return _this._ref.update(value); });
    };
    FirebaseDatabaseRef.prototype.remove = function () {
        var _this = this;
        return wrapPromise(function () { return _this._ref.remove(); });
    };
    FirebaseDatabaseRef.prototype.transaction = function (transactionUpdate, applyLocally) {
        var _this = this;
        return wrapPromise(function () { return new Promise(function (resolve, reject) { return _this._ref.transaction(transactionUpdate, function (err, committed, snapshot) { return err ? reject(err) : resolve({ committed: committed, snapshot: snapshot }); }, applyLocally); }); });
    };
    return FirebaseDatabaseRef;
}(FirebaseQuery));
/**
 * A special object with information about the connection between client and server which can be
 * accessed by using `db.ref('.info')`.
 */
var InfoSchema = (function () {
    function InfoSchema() {
    }
    return InfoSchema;
}());
var FirebaseDatabase = (function () {
    function FirebaseDatabase(db) {
        this.db = db;
    }
    FirebaseDatabase.prototype.ref = function (path) {
        return FirebaseDatabaseRef.create(null, this.db.ref(path));
    };
    return FirebaseDatabase;
}());
/**
 * A collection of special constants which can be used when writing data. Their values will be
 * substituted on the server with server generated values.
 * E.g {@link FirebaseDatabase.ServerValue.TIMESTAMP} will be substituted for the server time
 * when committing a write.
 */
FirebaseDatabase.ServerValue = database.ServerValue;
FirebaseDatabase = __decorate$2([
    Injectable(),
    __metadata$1("design:paramtypes", [NativeFirebaseDatabase])
], FirebaseDatabase);

var lastAppId = 0;
var FirebaseApp = (function () {
    function FirebaseApp(config, injector) {
        this.config = config;
        this.injector = injector;
        config.name = config.name || "app-" + lastAppId++;
        this.nativeApp = initializeApp(config.options, config.name);
    }
    FirebaseApp.prototype.delete = function () {
        var _this = this;
        return wrapPromise(function () { return _this.nativeApp.delete(); });
    };
    /**
     * Get the with this {@link FirebaseApp} associated {@link FirebaseAuth} instance.
     */
    FirebaseApp.prototype.auth = function () {
        if (!this._auth) {
            var authInjector = ReflectiveInjector.resolveAndCreate([
                {
                    provide: NativeFirebaseAuth,
                    useValue: this.nativeApp.auth()
                },
                FirebaseAuth
            ], this.injector);
            this._auth = authInjector.get(FirebaseAuth);
        }
        return this._auth;
    };
    /**
     * Get the with this {@link FirebaseApp} associated {@link FirebaseDatabase}.
     *
     * The type parameter T is used to supply the schema of your database. If you do not want to
     * use one set it to `any`. Using a schema provides type safety when accessing the database.
     */
    FirebaseApp.prototype.database = function () {
        if (!this._database) {
            var databaseInjector = ReflectiveInjector.resolveAndCreate([
                {
                    provide: NativeFirebaseDatabase,
                    useValue: this.nativeApp.database()
                },
                FirebaseDatabase
            ], this.injector);
            this._database = databaseInjector.get(FirebaseDatabase);
        }
        return this._database;
    };
    return FirebaseApp;
}());

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/** @internal */
var lazyInvocation = true;
/** @internal */
function invokeLazy() {
    return lazyInvocation;
}
/**
 * Enable or disable lazy invocation of firebase operations.
 *
 * Since the library focuses on observables, all operations are invoked lazily as is usually the
 * case with observables. This means for example, calling `someRef.set({ foo: 'bar' })` will do
 * nothing without either subscribing to the returned observable or calling `toPromise()` on it.
 *
 * This is in contrast to the Firebase Web-API which starts the operation when the function is
 * called. It is possible to globally configure the library to behave like the native Firebase
 * Web-API by calling `setLazyInvocation(false)`
 *
 * @param lazy
 */
function setLazyInvocation(lazy) {
    lazyInvocation = lazy;
}
/** @internal */
function appFactory(injector, config) {
    return new FirebaseApp(config, injector);
}
/** @internal */
function authFactory(app) {
    return app.auth();
}
/** @internal */
function databaseFactory(app) {
    return app.database();
}
var FirebaseModule = FirebaseModule_1 = (function () {
    function FirebaseModule() {
    }
    /**
     * Provides a firebase app which will be be injected for {@link FirebaseApp}. Further the app's
     * {@link FirebaseAuth} and {@link FirebaseDatabase} can be injected in the same way.
     * @param config Firebase app config.
     */
    FirebaseModule.primaryApp = function (config) {
        return {
            ngModule: FirebaseModule_1,
            providers: [
                {
                    provide: 'firebase-config-' + config.options.apiKey,
                    useValue: config
                },
                {
                    provide: FirebaseApp,
                    useFactory: appFactory,
                    deps: [Injector, 'firebase-config-' + config.options.apiKey]
                },
                {
                    provide: FirebaseAuth,
                    useFactory: authFactory,
                    deps: [FirebaseApp]
                },
                {
                    provide: FirebaseDatabase,
                    useFactory: databaseFactory,
                    deps: [FirebaseApp]
                }
            ]
        };
    };
    /**
     * Provides a {@link FirebaseApp} which will be injected for token.
     *
     * @param token Token for which {@link FirebaseApp} will be injected.
     * @param config Firebase app config.
     */
    FirebaseModule.secondaryApp = function (token, config) {
        return {
            ngModule: FirebaseModule_1,
            providers: [
                {
                    provide: 'firebase-config-' + config.options.apiKey,
                    useValue: config
                },
                {
                    provide: token,
                    useFactory: appFactory,
                    deps: [Injector, 'firebase-config-' + config.options.apiKey,]
                }
            ]
        };
    };
    return FirebaseModule;
}());
FirebaseModule = FirebaseModule_1 = __decorate([
    NgModule({})
], FirebaseModule);
var FirebaseModule_1;

var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var GoogleAuthProvider = (function (_super) {
    __extends$2(GoogleAuthProvider, _super);
    function GoogleAuthProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return GoogleAuthProvider;
}(auth.GoogleAuthProvider));
var FacebookAuthProvider = (function (_super) {
    __extends$2(FacebookAuthProvider, _super);
    function FacebookAuthProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return FacebookAuthProvider;
}(auth.FacebookAuthProvider));
var GithubAuthProvider = (function (_super) {
    __extends$2(GithubAuthProvider, _super);
    function GithubAuthProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return GithubAuthProvider;
}(auth.GithubAuthProvider));
var EmailAuthProvider = (function (_super) {
    __extends$2(EmailAuthProvider, _super);
    function EmailAuthProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return EmailAuthProvider;
}(auth.EmailAuthProvider));
var TwitterAuthProvider = (function (_super) {
    __extends$2(TwitterAuthProvider, _super);
    function TwitterAuthProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return TwitterAuthProvider;
}(auth.TwitterAuthProvider));

export { FirebaseModule, setLazyInvocation, DataSnapshotObservable, FirebaseApp, FirebaseAuth, Event, FirebaseQuery, FirebaseDatabaseRefConfig, FirebaseDatabaseRef, InfoSchema, FirebaseDatabase, FirebaseUserCredential, FirebaseUser, GoogleAuthProvider, FacebookAuthProvider, GithubAuthProvider, EmailAuthProvider, TwitterAuthProvider };
//# sourceMappingURL=angular-firebase.js.map
