import { InjectionToken, Injector, ModuleWithProviders } from '@angular/core';
import { FirebaseApp, FirebaseAppConfig } from './firebase-app.service';
/**
 * Enable or disable lazy invocation of firebase operations.
 *
 * Since the library focuses on observables, all operations are invoked lazily as is usually the
 * case with observables. This means for example, calling `someRef.set({ foo: 'bar' })` will do
 * nothing without either subscribing to the returned observable or calling `toPromise()` on it.
 *
 * This is in contrast to the Firebase Web-API which starts the operation when the function is
 * called. It is possible to globally configure the library to behave like the native Firebase
 * Web-API by calling `setLazyInvocation(false)`
 *
 * @param lazy
 */
export declare function setLazyInvocation(lazy: boolean): void;
export declare class FirebaseModule {
    /**
     * Provides a firebase app which will be be injected for {@link FirebaseApp}. Further the app's
     * {@link FirebaseAuth} and {@link FirebaseDatabase} can be injected in the same way.
     * @param config Firebase app config.
     */
    static primaryApp(config: FirebaseAppConfig): ModuleWithProviders;
    /**
     * Provides a {@link FirebaseApp} which will be injected for token.
     *
     * @param token Token for which {@link FirebaseApp} will be injected.
     * @param config Firebase app config.
     */
    static secondaryApp(token: InjectionToken<FirebaseApp>, config: FirebaseAppConfig): {
        ngModule: typeof FirebaseModule;
        providers: ({
            provide: string;
            useValue: FirebaseAppConfig;
        } | {
            provide: InjectionToken<FirebaseApp>;
            useFactory: (injector: Injector, config: FirebaseAppConfig) => FirebaseApp;
            deps: (string | typeof Injector)[];
        })[];
    };
}
