import { database } from 'firebase';
import { Observable } from 'rxjs/Observable';
export interface ExtendedDataSnapshot extends database.DataSnapshot {
    prevKey?: string;
}
export declare class DataSnapshotObservable<T> extends Observable<ExtendedDataSnapshot> {
    exists(): Observable<boolean>;
    children(): Observable<DataSnapshotObservable<T[keyof T]>>;
    /**
     * This operator takes the result of .val() for all children of the snapshot and emits
     * them as an array.
     * Contents of source snapshot:
     * ```
     * {
     *  childA: { prop: 'Hello' },
     *  childB: { prop: 'World!' },
     * }
     * ```
     * Result of operator:
     * ```
     * [
     *  { prop: 'Hello' },
     *  { prop: 'World!' },
     * ]
     * ```
     * @returns {Observable<C[]>}
     */
    toValArray(): Observable<T[keyof T][]>;
    values(): Observable<T[keyof T][]>;
    keys(): Observable<string[]>;
    list(): Observable<{
        val: T[keyof T];
        key: string;
    }[]>;
    entry(): Observable<{
        val: T;
        key: string;
    }>;
    key(): Observable<string>;
    /**
     * When listening to events such as {@link Event.ChildMoved} the snapshot includes
     * the key of the child before this snapshots one. This operator maps to this key.
     * @returns {Observable<string>}
     */
    prevKey(): Observable<string>;
    val(): Observable<T>;
    getPriority(): Observable<number | string>;
    exportVal(): Observable<T>;
    hasChild(path: string): Observable<boolean>;
    hasChildren(): Observable<boolean>;
    numChildren(): Observable<number>;
    child<P extends keyof T>(path: P): DataSnapshotObservable<T[P]>;
}
